mod create;

use crate::entity::{Entity, Index, ConstraintVisitor};

/// How we describe an entire schema
#[derive(Debug)]
pub struct Schema {
    drop: Vec<String>,
    create: Vec<String>,
}

impl Schema {
    pub fn new() -> Self {
        Self {
            drop: Vec::new(),
            create: Vec::new(),
        }
    }

    pub fn entity<E: Entity>(mut self) -> Self {
        let (drop, create) = create::sql_for_table::<E>();
        self.drop.push(drop);
        self.create.push(create);

        struct Visitor<'l> {
            drop: &'l mut Vec<String>,
            create: &'l mut Vec<String>,
        }

        impl<'l> ConstraintVisitor for Visitor<'l> {
            fn index<I: Index>(&mut self) {
                let (drop, create) = create::sql_for_index::<I>();
                self.drop.push(drop);
                self.create.push(create);
            }
        }

        E::constraints(&mut Visitor { drop: &mut self.drop, create: &mut self.create });

        self
    }

    pub fn index<I: Index>(mut self) -> Self {
        let (drop, create) = create::sql_for_index::<I>();
        self.drop.push(drop);
        self.create.push(create);
        self
    }

    pub fn add<E: Entity>(self) -> Self {
        self.entity::<E>()
    }

    pub fn drop(&self) -> &Vec<String> {
        &self.drop
    }
    pub fn create(&self) -> &Vec<String> {
        &self.create
    }
}

impl Default for Schema {
    fn default() -> Self {
        Self::new()
    }
}
